#ifndef ERS_ERR_MSGTH
#define ERS_ERR_MSGTH 1
/*  Message table include file
 *  Generated by /Users/drama/release/messgen/r3_16/macosx_x86_64/messgen utility
 *  from input file ../Ers_Err.msg at Tue Dec  8 19:01:48 2020
  */
static MessTabType MessFacTab_ERS[] = {
    { 2, "NOSPACE", 
        "The ERS system error table is full" },
    { 3, "ACTIVE", 
        "The ERS system is already active" },
    { 4, "NOTACTIVE", 
        "The ERS system is not active" },
    { 5, "TOOMANPOPS", 
        "ErsPop was called more times then ErsPush" },
    { 6, "TASKVARERR", 
        "Error adding or deleting task a variable" },
    { 7, "MALLOCERR", 
        "Malloc error" },
    { 8, "INITIALISED", 
        "Ers already initialised" },
    { 9, "NOTINITIALISED", 
        "Ers not initialised" },
    { 10, "FLUSHING", 
        "Operation not permissible while flushing" },
    { 11, "INTHAND", 
        "Operation (ErsClear/ErsFlush) is not permissible while ERS is in interrupt mode" },
    {0, 0, 0 }};

static MessFacType MessFac_ERS = 
  { 1827, "ERS", MessFacTab_ERS, 0 };

#endif
