#ifndef SDS_ERRH
#define SDS_ERRH 1
/* C language ERROR include file
 * Generated by /Users/drama/release/messgen/r3_16/macosx_x86_64/messgen utility
 * from input file ../sds_err.msg at Tue Dec  8 19:19:17 2020
 */

/* Facility number */
#define SDS__FACNUM 1824

/* OK Status symbol */
#define SDS__OK 0

/* Illegal Identifier */
#define SDS__BADID 0xF208322

/* Insufficient Memory */
#define SDS__NOMEM 0xF20832A

/* Object is not a structure */
#define SDS__NOTSTRUCT 0xF208332

/* No such item */
#define SDS__NOITEM 0xF20833A

/* Object is not a primitive */
#define SDS__NOTPRIM 0xF208342

/* Data too long */
#define SDS__TOOLONG 0xF20834A

/* Data value is undefined */
#define SDS__UNDEFINED 0xF208352

/* Object name is too long */
#define SDS__LONGNAME 0xF20835A

/* Too much extra data */
#define SDS__EXTRA 0xF208362

/* Invalid type code */
#define SDS__INVCODE 0xF20836A

/* Invalid dimensions */
#define SDS__INVDIMS 0xF208372

/* Not a structure array */
#define SDS__NOTARRAY 0xF20837A

/* Indices invalid */
#define SDS__INDEXERR 0xF208382

/* Object cannot be deleted */
#define SDS__ILLDEL 0xF20838A

/* Not a valid SDS object */
#define SDS__NOTSDS 0xF208392

/* Not a top level object */
#define SDS__NOTTOP 0xF20839A

/* Illegal operation on external object */
#define SDS__EXTERN 0xF2083A2

/* Object is not an array */
#define SDS__NOTARR 0xF2083AA

/* Object created by an incompatible SDS version */
#define SDS__VERSION 0xF2083B2

/* Error opening file */
#define SDS__FOPEN 0xF2083BA

/* Error writing to a file */
#define SDS__FWRITE 0xF2083C2

/* Error reading a file */
#define SDS__FREAD 0xF2083CA

/* Object has incorrect type */
#define SDS__TYPE 0xF2083D2

/* Timeout on semaphore access */
#define SDS__TIMEOUT 0xF2083DA

/* Invalid primitive type code */
#define SDS__INVPRIMTYPE 0xF2083E2

/* Invalid structure definition */
#define SDS__INVSTRUCTDEF 0xF2083EA

/* Parsing syntax error */
#define SDS__SYNTAX 0xF2083F2

/* Invalid structure */
#define SDS__INVSTRUCTURE 0xF2083FA

/* Invalid typedef type */
#define SDS__INVTYPETYPE 0xF208402

/* Multipled defined structure */
#define SDS__STRUCTMULTDEF 0xF20840A

/* Invalid integer */
#define SDS__INVINT 0xF208412

/* Invalid type */
#define SDS__INVTYPE 0xF20841A

/* Invalid input */
#define SDS__INVINPUT 0xF208422

/* Array of structures */
#define SDS__STRUCTARRAY 0xF20842A

/* Maximum number of dimensions */
#define SDS__MAXDIMS 0xF208432

/* Empty input file */
#define SDS__NOINPUT 0xF20843A

/* Error in command arguments */
#define SDS__TCL_ARGERR 0xF208442

/* Unspecified error in Tcl command */
#define SDS__TCL_CMDERR 0xF20844A

/* Error in Tcl Command Action */
#define SDS__TCL_CMD 0xF208452

/* SDS Test failure */
#define SDS__TESTERR 0xF20845A

/* Sds Item definition does not match the definition expected by the appliaction code */
#define SDS__INVITEM 0xF208462

/* SdsCheck() indicates a SDS data leak */
#define SDS__CHK_LEAK 0xF20846A

/* SdsCheck() indicates an unexpected release of an SDS id */
#define SDS__CHK_RELEASED 0xF208472

/* SdsFreeIdAndCheck() indicates SDS misuse (missing SdsDelete() or SdsReadFree() call) */
#define SDS__CHK_FREE 0xF20847A

/* SDS Semaphore/Mutex initialisation error */
#define SDS__SEM_INIT_ERR 0xF208482

/* Failed to take SDS Semaphore/Mutex */
#define SDS__SEM_TAKE_ERR 0xF20848A

/* Ambigous type name - when compling C to SDS structure */
#define SDS__AMBIGIOUS_TYPE 0xF208492

#endif
