#ifndef ARG_ERR_MSGTH
#define ARG_ERR_MSGTH 1
/*  Message table include file
 *  Generated by /Users/drama/release/messgen/r3_16/macosx_x86_64/messgen utility
 *  from input file ../arg_err.msg at Tue Dec  8 19:27:52 2020
  */
static MessTabType MessFacTab_ARG[] = {
    { 100, "CNVERR", 
        "Type Conversion Error" },
    { 101, "NOTSCALAR", 
        "Object is not a scalar" },
    { 102, "NOTSTRING", 
        "Object is not a string" },
    { 103, "MALLOCERR", 
        "Malloc failed" },
    {0, 0, 0 }};

static MessFacType MessFac_ARG = 
  { 1825, "ARG", MessFacTab_ARG, 0 };

#endif
