#ifndef ARG_ERRH
#define ARG_ERRH 1
/* C language ERROR include file
 * Generated by /Users/drama/release/messgen/r3_16/macosx_x86_64/messgen utility
 * from input file ../arg_err.msg at Tue Dec  8 19:27:52 2020
 */

/* Facility number */
#define ARG__FACNUM 1825

/* OK Status symbol */
#define ARG__OK 0

/* Type Conversion Error */
#define ARG__CNVERR 0xF218322

/* Object is not a scalar */
#define ARG__NOTSCALAR 0xF21832A

/* Object is not a string */
#define ARG__NOTSTRING 0xF218332

/* Malloc failed */
#define ARG__MALLOCERR 0xF21833A

#endif
