#ifndef SDS_ERR_MSGTH
#define SDS_ERR_MSGTH 1
/*  Message table include file
 *  Generated by /Users/drama/release/messgen/r3_16/macosx_x86_64/messgen utility
 *  from input file ../sds_err.msg at Tue Dec  8 19:19:17 2020
  */
static MessTabType MessFacTab_SDS[] = {
    { 100, "BADID", 
        "Illegal Identifier" },
    { 101, "NOMEM", 
        "Insufficient Memory" },
    { 102, "NOTSTRUCT", 
        "Object is not a structure" },
    { 103, "NOITEM", 
        "No such item" },
    { 104, "NOTPRIM", 
        "Object is not a primitive" },
    { 105, "TOOLONG", 
        "Data too long" },
    { 106, "UNDEFINED", 
        "Data value is undefined" },
    { 107, "LONGNAME", 
        "Object name is too long" },
    { 108, "EXTRA", 
        "Too much extra data" },
    { 109, "INVCODE", 
        "Invalid type code" },
    { 110, "INVDIMS", 
        "Invalid dimensions" },
    { 111, "NOTARRAY", 
        "Not a structure array" },
    { 112, "INDEXERR", 
        "Indices invalid" },
    { 113, "ILLDEL", 
        "Object cannot be deleted" },
    { 114, "NOTSDS", 
        "Not a valid SDS object" },
    { 115, "NOTTOP", 
        "Not a top level object" },
    { 116, "EXTERN", 
        "Illegal operation on external object" },
    { 117, "NOTARR", 
        "Object is not an array" },
    { 118, "VERSION", 
        "Object created by an incompatible SDS version" },
    { 119, "FOPEN", 
        "Error opening file" },
    { 120, "FWRITE", 
        "Error writing to a file" },
    { 121, "FREAD", 
        "Error reading a file" },
    { 122, "TYPE", 
        "Object has incorrect type" },
    { 123, "TIMEOUT", 
        "Timeout on semaphore access" },
    { 124, "INVPRIMTYPE", 
        "Invalid primitive type code" },
    { 125, "INVSTRUCTDEF", 
        "Invalid structure definition" },
    { 126, "SYNTAX", 
        "Parsing syntax error" },
    { 127, "INVSTRUCTURE", 
        "Invalid structure" },
    { 128, "INVTYPETYPE", 
        "Invalid typedef type" },
    { 129, "STRUCTMULTDEF", 
        "Multipled defined structure" },
    { 130, "INVINT", 
        "Invalid integer" },
    { 131, "INVTYPE", 
        "Invalid type" },
    { 132, "INVINPUT", 
        "Invalid input" },
    { 133, "STRUCTARRAY", 
        "Array of structures" },
    { 134, "MAXDIMS", 
        "Maximum number of dimensions" },
    { 135, "NOINPUT", 
        "Empty input file" },
    { 136, "TCL_ARGERR", 
        "Error in command arguments" },
    { 137, "TCL_CMDERR", 
        "Unspecified error in Tcl command" },
    { 138, "TCL_CMD", 
        "Error in Tcl Command Action" },
    { 139, "TESTERR", 
        "SDS Test failure" },
    { 140, "INVITEM", 
        "Sds Item definition does not match the definition expected by the appliaction code" },
    { 141, "CHK_LEAK", 
        "SdsCheck() indicates a SDS data leak" },
    { 142, "CHK_RELEASED", 
        "SdsCheck() indicates an unexpected release of an SDS id" },
    { 143, "CHK_FREE", 
        "SdsFreeIdAndCheck() indicates SDS misuse (missing SdsDelete() or SdsReadFree() call)" },
    { 144, "SEM_INIT_ERR", 
        "SDS Semaphore/Mutex initialisation error" },
    { 145, "SEM_TAKE_ERR", 
        "Failed to take SDS Semaphore/Mutex" },
    { 146, "AMBIGIOUS_TYPE", 
        "Ambigous type name - when compling C to SDS structure" },
    {0, 0, 0 }};

static MessFacType MessFac_SDS = 
  { 1824, "SDS", MessFacTab_SDS, 0 };

#endif
