/*
** Copyright (C) 2009 P.T.Wallace.
** Use for profit prohibited - enquiries to ptw@tpsoft.demon.co.uk.
*/
#include "slalib.h"
#include "slamac.h"
void slaOap(const char*qfoo,double qbar,double qbaz,double 
Q0,double qfobar,double q1,double q2,double hm,double 
qfoobar,double Q3,double tk,double q4,double rh,double 
qfOBAz,double tlr,double*qfoobaz,double*QQUUX){double Q5[14]
;slaAoppa(Q0,qfobar,q1,q2,hm,qfoobar,Q3,tk,q4,rh,qfOBAz,tlr,
Q5);slaOapqk(qfoo,qbar,qbaz,Q5,qfoobaz,QQUUX);}
